﻿module Eraser

open System

//====================================================
// Подробное описание исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================


type Coordinates = { mutable row: int; mutable col: int }


type Rectangle(numrows: int, numbercols: int) as this =
    let symbol = "●"

    let arrayPos: Coordinates[,]=
        Array2D.create numrows numbercols { row = 0; col = 0 }

    let cursorpos pos =
        Console.CursorLeft <- pos.col
        Console.CursorTop <- pos.row

    let drawsymbols pos =
        cursorpos pos
        printf "%s" symbol

    let erasesymbols pos =
        cursorpos pos
        printf "%s" " "

    let erase direction =

        match direction with
        | 1 -> // erase left for right

            (*for i = 0 to numrows - 1 do
                let leftcol = arrayPos.[i, 0]
                erasesymbols leftcol*)
            for i = 0 to this.Left.Length - 1 do
                erasesymbols this.Left[i]

        | 2 -> // erase right for left
            for i = 0 to numrows - 1 do
                let rightcol = arrayPos.[i, numbercols - 1]
                erasesymbols rightcol
        | 3 -> // erase top for down
            for i = 0 to numbercols - 1 do
                let toprow = arrayPos.[0, i]
                erasesymbols toprow
        | 4 -> // erase bottom for up
            for i = 0 to numbercols - 1 do
                let bottomcol = arrayPos.[numrows - 1, i]
                erasesymbols bottomcol
        | _ -> ()

        

    let lastIndex dimension = arrayPos.GetLength(dimension) - 1

    do
        for row = 0 to numrows - 1 do
            for col = 0 to numbercols - 1 do
                arrayPos.[row, col] <- { row = row; col = col }
                drawsymbols arrayPos.[row, col]

    // Получение координат прямоугольника.
    member x.Pos = arrayPos

    // Крайние ряды и столбцы для проверки касания.
    member x.Right = arrayPos[*, lastIndex(1)]
    member x.Left: Coordinates[] = arrayPos[*, 0]
    member x.Top = arrayPos[0, *]
    member x.Bottom = arrayPos[lastIndex(0), *]

    // Передвижение на один шаг в указанном направлении.
    member x.Move direction =

        let mutable x = 0
        let mutable y = 0

        // Определитель направления движения.
        match direction with
        // направо
        | 1 when arrayPos.[0, lastIndex (1)].col < (Console.WindowWidth - 1) -> x <- 1
        // налево
        | 2 when arrayPos.[0, 0].col > 0 -> x <- -1
        // вниз
        | 3 when arrayPos.[lastIndex (0), 0].row < Console.WindowHeight -> y <- 1
        // вверх
        | 4 when arrayPos.[0, 0].row > 0 -> y <- -1
        | _ -> ()

        if x <> 0 || y <> 0 then
            erase direction

            // Каждый символ передвигаем на шаг в указанном направлении.
            arrayPos
            |> Array2D.iter
                (fun pos ->
                    pos.col <- pos.col + x
                    pos.row <- pos.row + y
                    drawsymbols pos)

